/*
 * The software is provided under the Copyright, 2008.
 * Software Tools Technology Group, Computer Science Dept.,
 * Worcester Polytechnic Institute, sttg@cs.wpi.edu
 *
 * The program is provided under the terms and conditions of
 * the Eclipse Public License Version 1.0 ("EPL"). A copy of the EPL
 * is available at http://www.eclipse.org/org/documents/epl-v10.php.
 *
 * $Id:$
 */
package edu.webfoot.vnc;

import org.eclipse.jface.preference.*;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.IWorkbench;

import edu.webfoot.vnc.Activator;

/**
 * This class represents a preference page that is contributed to the
 * Preferences dialog. By subclassing <samp>FieldEditorPreferencePage</samp>,
 * we can use the field support built into JFace that allows us to create a page
 * that is small and knows how to save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They are stored in the
 * preference store that belongs to the main plug-in class. That way,
 * preferences can be accessed directly via the preference store.
 * 
 * @author Ken  Breeman (kbreeman@wpi.edu)
 * @author Eric Griffel (sniffels@wpi.edu)
 */

public class PreferencePage extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage {

	public PreferencePage() {
		super(GRID);
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		setDescription("VNC Preference Page");
	}

	/**
	 * Creates the field editors. Field editors are abstractions of the common
	 * GUI blocks needed to manipulate various types of preferences. Each field
	 * editor knows how to save and restore itself.
	 */
	public void createFieldEditors() {
		StringFieldEditor password = new StringFieldEditor(
				PreferenceConstants.pass, "The &password for the host ",
				getFieldEditorParent());
		password.setEmptyStringAllowed(true);
		password.setTextLimit(8);
		addField(password);

		StringFieldEditor host = new StringFieldEditor(
				PreferenceConstants.hostName, "The &name of the host ",
				getFieldEditorParent());
		host.setEmptyStringAllowed(false);
		host.setTextLimit(32);
		addField(host);

		IntegerFieldEditor port = new IntegerFieldEditor(
				PreferenceConstants.port, "The p&ort that the host acts upon ",
				getFieldEditorParent());
		port.setValidRange(1, 65535);
		addField(port);

		IntegerFieldEditor refresh = new IntegerFieldEditor(
				PreferenceConstants.refresh,
				"The update &rate (number of redraws/sec)",
				getFieldEditorParent());
		refresh.setValidRange(1, 10000);
		addField(refresh);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

}