/*
 * The software is provided under the Copyright, 2008.
 * Software Tools Technology Group, Computer Science Dept.,
 * Worcester Polytechnic Institute, sttg@cs.wpi.edu
 *
 * The program is provided under the terms and conditions of
 * the Eclipse Public License Version 1.0 ("EPL"). A copy of the EPL
 * is available at http://www.eclipse.org/org/documents/epl-v10.php.
 *
 * $Id:$
 */
package edu.webfoot.vnc.editor;

import org.eclipse.swt.SWTException;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

/**
 * A Runnable class that continually updates the image that the plug-in view
 * is displaying. Refers to the client for the image.
 * 
 * @author Ken  Breeman (kbreeman@wpi.edu)
 * @author Eric Griffel (sniffels@wpi.edu)
 */
public class AutoRefresh implements Runnable {
	private boolean keepRunning;
	private VNCEditor vncv;
	
	/**
	 * Private constructor
	 */
	public AutoRefresh(VNCEditor vncv) {
		keepRunning = true;
		this.vncv = vncv;		
	}

	/**
	 * Main method for the runnable. Updates view every 100ms. Terminates when
	 * this.stopRunning is invoked. Once terminated, this.restart must be
	 * invoked before runnable can be run again.
	 */
	public void run() {
		//System.out.println("Reached initial point of AutoRefresh.run()");
		
		// This section prevents a null pointer exception.
		try {
			this.vncv.getImageCanvas().setImageData(edu.webfoot.vnc.Activator.getImageDescriptor("icons/vnc.bmp").createImage().getImageData());
		} catch(Exception e) {
			return;
		}
		
		
		while (keepRunning) {
			//System.out.println("Start of AutoRefresh loop");
			
			int pixels[];
			if ((pixels = this.vncv.getClient().getImage()) != null) {
				// Sets image data
				int width = this.vncv.getClient().getWidth();
				int height = this.vncv.getClient().getHeight();
				int depth = this.vncv.getClient().getDepth();
				//PixelFormat currentPixelFormat = this.vncv.getClient().getPixelFormat();
				
				//Correctly interprets the VNC color.
				PaletteData paletteData = new PaletteData(0xFF0000,0x00FF00,0x0000FF);
				
				ImageData id = new ImageData(width, height, depth, paletteData);
	
				for (int y = 0; y < height; y++) {
					for (int x = 0; x < width; x++) {
						id.setPixel(x, y, pixels[y * width + x]);
					}
				}
	
				//System.out.println("Set pixels[]");
				
				this.vncv.getImageCanvas().setImageData(id);
				
				// Redraw
				//System.out.println("Redraw now");
				try { // try/catch so loop will continue to run even if an error occurs
					this.vncv.getImageCanvas().getDisplay().asyncExec(new Runnable() {
						public void run() {
							IWorkbenchPage iwp = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
							IEditorReference ier[] = iwp.getEditorReferences();
							VNCEditor vncv = null;
							for (int c = 0; c < ier.length; c++) {
								if (ier[c].getId().equals("gnu.editor.VNCViewer")) {
									vncv = (VNCEditor)ier[c].getEditor(false);
									continue;
								}
							}
							
							vncv.getImageCanvas().redraw();
						}
					});
				} catch (SWTException e) {
					//e.printStackTrace();
					return;
				}
			}

			try {
				Thread.sleep(1000/edu.webfoot.vnc.Activator.getDefault().getPreferenceStore().getInt(edu.webfoot.vnc.PreferenceConstants.refresh), 0);
			} catch (InterruptedException e) {
			}
			
			//System.out.println("End of AutoRefresh loop");
		}
		
		//System.out.println("Reached terminal point of AutoRefresh.run()");

		return;
	}

	/**
	 * Terminates this.run method.
	 */
	public void stopRunning() {
		keepRunning = false;
		//System.out.println("Terminating AutoRefresh");
	}

	/**
	 * Re-initializes the this.run method so it can be run again.
	 */
	public void restart() {
		keepRunning = true;
	}
}
