/*
 * The software is provided under the Copyright, 2008.
 * Software Tools Technology Group, Computer Science Dept.,
 * Worcester Polytechnic Institute, sttg@cs.wpi.edu
 *
 * The program is provided under the terms and conditions of
 * the Eclipse Public License Version 1.0 ("EPL"). A copy of the EPL
 * is available at http://www.eclipse.org/org/documents/epl-v10.php.
 *
 * $Id:$
 */
package edu.webfoot.vnc.editor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

/**
 * A simple Image canvas for SWT images.
 * Portable and lightweight, does not depend on outside tools or specifis OS's.
 * 
 * @author Ken  Breeman (kbreeman@wpi.edu)
 * @author Eric Griffel (sniffels@wpi.edu)
 */
public class ImageCanvas extends Canvas implements PaintListener, MouseListener {

	private Image img;
	private Image currentImage;
	private int width;
	private int height;
	private MouseListener mouseListenerProxy;
	
	public ImageCanvas(Composite parent, int style, int w, int h) {
		super(parent, style | SWT.NO_BACKGROUND);
		this.width = w;
		this.height = h;
		
		this.setLayout(new FillLayout());
		this.addPaintListener(this);
		
		this.addMouseListener(this);
	}
	
	public void paintControl(PaintEvent e) {
		if(this.getImage() == null) {
			return;
		} else {
			if(currentImage != null && currentImage.isDisposed() == false) {
				currentImage.dispose();
			}
			currentImage = this.getImage();
		}
		
		GC gc = e.gc;
		gc.drawImage(currentImage, 0, 0);
	}
	
	public Point computeSize(int wHint, int hHint, boolean changed) {
		return new Point(this.width, this.height);
	}
	
	private synchronized Image getImage() {
		return this.img;
	}
	
	public synchronized void setImageData(ImageData id) {
		if(this.img != null && this.img.isDisposed() == false) {
			this.img.dispose();
		}
		this.img = new Image(this.getDisplay(), id);
	}

	public void mouseDoubleClick(MouseEvent e) {
		this.mouseListenerProxy.mouseDoubleClick(e);
	}

	public void mouseDown(MouseEvent e) {
		this.mouseListenerProxy.mouseDown(e);
	}

	public void mouseUp(MouseEvent e) {
		this.mouseListenerProxy.mouseUp(e);
	}
	
	public void setMouseListener(MouseListener ml) {
		this.mouseListenerProxy = ml;
	}

}
