/*
 * The software is provided under the Copyright, 2008.
 * Software Tools Technology Group, Computer Science Dept.,
 * Worcester Polytechnic Institute, sttg@cs.wpi.edu
 *
 * The program is provided under the terms and conditions of
 * the Eclipse Public License Version 1.0 ("EPL"). A copy of the EPL
 * is available at http://www.eclipse.org/org/documents/epl-v10.php.
 *
 * $Id:$
 */
package edu.webfoot.vnc.editor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ScrollBar;

/**
 * A simple Scrolling canvas in SWT.
 * Allows the contents to scroll without needing to know any details.
 * 
 * @author Ken  Breeman (kbreeman@wpi.edu)
 * @author Eric Griffel (sniffels@wpi.edu)
 */
public class ScrollingCanvas extends ScrolledComposite {

	private int width;
	private int height;
	
	public ScrollingCanvas(Composite parent, int style, int w, int h) {
		super(parent, style | SWT.V_SCROLL | SWT.H_SCROLL | SWT.NO_BACKGROUND);
		
		this.width = w;
		this.height = h;
		
		this.setMinSize(width, height);
		this.setExpandHorizontal(true);
		this.setExpandVertical(true);
		
		ScrollBar hBar = this.getHorizontalBar();
		hBar.setIncrement(width/5);
		hBar.setPageIncrement(width/5);
		hBar.setMinimum(0);
		hBar.setMaximum(width);
		
		ScrollBar vBar = this.getVerticalBar();
		vBar.setIncrement(height/5);
		vBar.setPageIncrement(height/5);
		vBar.setMinimum(0);
		vBar.setMaximum(height);
		
		hBar.setEnabled(true);
		vBar.setEnabled(true);
	}

	public Point computeSize(int wHint, int hHint, boolean changed) {
		return new Point(width, height);
	}

}
