/*
 * The software is provided under the Copyright, 2008.
 * Software Tools Technology Group, Computer Science Dept.,
 * Worcester Polytechnic Institute, sttg@cs.wpi.edu
 *
 * The program is provided under the terms and conditions of
 * the Eclipse Public License Version 1.0 ("EPL"). A copy of the EPL
 * is available at http://www.eclipse.org/org/documents/epl-v10.php.
 *
 * $Id:$
 */
package edu.webfoot.vnc.editor;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import gnu.rfb.KeySym;
import gnu.vnc.VNCViewer;

/**
 * This editor is used to display the desktop of a remote system.
 * It uses ScrollingCanvas and ImageCanvas for the graphics.
 * AutoRefresh takes care of re-sampling the data and redrawing.
 * 
 * Keyboard and Mouse events are listened for and sent on the remote system.
 * 
 * @author Ken  Breeman (kbreeman@wpi.edu)
 * @author Eric Griffel (sniffels@wpi.edu)
 */
public class VNCEditor extends EditorPart implements KeyListener,
		MouseListener, FocusListener {
	
	public static String newViewerId = ""; //hack :(
	
	private ScrollingCanvas scrollCanvas;
	private ImageCanvas imageCanvas;
	private gnu.vnc.VNCViewer client;
	private AutoRefresh ar;
	private String viewerId;
	private boolean hasFocus;

	public VNCEditor() {
		super();
	}

	public void dispose() {
		super.dispose();
		
		ar.stopRunning();
		this.scrollCanvas.dispose();
		this.imageCanvas.dispose();
		VNCSessionManager.getInstance().killSession(client.getServerHost());
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
	}

	@Override
	public void doSaveAs() {
	}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
		// setTitle("VNC Viewer Title");
	}

	@Override
	public boolean isDirty() {
		return false;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void createPartControl(Composite parent) {
		// Initializes editor window
		parent.setData(this);
		setTitleToolTip("VNC Connection Editor");
		
		setTitleImage(edu.webfoot.vnc.Activator.getImageDescriptor("icons/vnc.bmp").createImage());

		while(this.viewerId == null) {
			if(VNCEditor.newViewerId != "") {
				this.viewerId = VNCEditor.newViewerId;
				VNCEditor.newViewerId = "";
			}
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				
			}
		}
		
		if(this.client == null) {
			try {
				this.client = VNCSessionManager.getInstance().getSession(this.viewerId);
				if(this.client == null) {
					this.dispose();
					return;
				}
				this.setTitle(this.viewerId);
				
			} catch (Exception e) {
				//MessageDialog.openError(null, "Client initialization error", e.getMessage());
				this.dispose();
				return;
			}
		}
		
		//Wait until we are connected and running.
		while(this.client.getImage() == null) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
			}
		}
		
		this.setScrollCanvas(new ScrollingCanvas(parent, SWT.None, client.getWidth(), client.getHeight() ));
		this.setImageCanvas(new ImageCanvas(this.getScrollCanvas(), SWT.NONE, client.getHeight(), client.getHeight()));
		this.getScrollCanvas().setContent(this.getImageCanvas());
		
		
		// Listeners
		this.getImageCanvas().setMouseListener(this);
		this.getImageCanvas().addKeyListener(this);
		this.getImageCanvas().addFocusListener(this);
//		parent.addFocusListener(this);
	}

	@Override
	public void setFocus() {
		this.getImageCanvas().setFocus();
	}
	
	@SuppressWarnings("deprecation")
	public void setTitle(String title) {
		super.setTitle(title);
	}
	
	public AutoRefresh getRefresher() {
		return this.ar;
	}
	
	public void setRefresher(AutoRefresh ar) {
		this.ar = ar;
	}

	public void keyPressed(KeyEvent arg0) {
//		System.out.println("\n\nKeyCode: " + arg0.keyCode + " \n\n");
		
		// Send modifiers
		if ((arg0.stateMask & SWT.SHIFT) != 0) {
			this.client.sendKey(KeySym.ShiftL, true);
		}
		if ((arg0.stateMask & SWT.CTRL) != 0) {
			this.client.sendKey(KeySym.ControlL, true);
		}
		if ((arg0.stateMask & SWT.ALT) != 0) {
			this.client.sendKey(KeySym.AltL, true);
		}
		if ((arg0.stateMask & SWT.MOD1) != 0) {
			this.client.sendKey(KeySym.MetaL, true);
		}
		
		//Send unclear key codes.
		if(arg0.keyCode == SWT.ARROW_UP) {
			this.client.sendKey(KeySym.Up, true);
			this.client.sendKey(KeySym.Up, false);
		} else if(arg0.keyCode == SWT.ARROW_LEFT) {
			this.client.sendKey(KeySym.Left, true);
			this.client.sendKey(KeySym.Left, false);
		} else if(arg0.keyCode == SWT.ARROW_RIGHT) {
			this.client.sendKey(KeySym.Right, true);
			this.client.sendKey(KeySym.Right, false);
		} else if(arg0.keyCode == SWT.ARROW_DOWN) {
			this.client.sendKey(KeySym.Down, true);
			this.client.sendKey(KeySym.Down, false);
		
		} else if (arg0.keyCode == SWT.INSERT) {
			this.client.sendKey(KeySym.Insert, true);
			this.client.sendKey(KeySym.Insert, false);
		} else if (arg0.keyCode == SWT.DEL) {
			this.client.sendKey(KeySym.Delete, true);
			this.client.sendKey(KeySym.Delete, false);
		} else if (arg0.keyCode == SWT.HOME) {
			this.client.sendKey(KeySym.Home, true);
			this.client.sendKey(KeySym.Home, false);
		} else if (arg0.keyCode == SWT.END) {
			this.client.sendKey(KeySym.End, true);
			this.client.sendKey(KeySym.End, false);
		} else if (arg0.keyCode == SWT.PAGE_UP) {
			this.client.sendKey(KeySym.PageUp, true);
			this.client.sendKey(KeySym.PageUp, false);
		} else if (arg0.keyCode == SWT.PAGE_DOWN) {
			this.client.sendKey(KeySym.PageDown, true);
			this.client.sendKey(KeySym.PageDown, false);
			
		} else if (arg0.keyCode == SWT.CR) {
			this.client.sendKey(KeySym.Return, true);
			this.client.sendKey(KeySym.Return, false);
		} else if (arg0.keyCode == SWT.LF) {
			this.client.sendKey(KeySym.Return, true);
			this.client.sendKey(KeySym.Return, false);
		} else if (arg0.keyCode == SWT.BS) {
			this.client.sendKey(KeySym.BackSpace, true);
			this.client.sendKey(KeySym.BackSpace, false);
		} else if (arg0.keyCode == SWT.ESC) {
			this.client.sendKey(KeySym.Escape, true);
			this.client.sendKey(KeySym.Escape, false);
			
			
		} else {
			//Send key
			this.client.sendKey(arg0.keyCode, true);
		}
		

		// Clear modifiers
		if ((arg0.stateMask & SWT.SHIFT) != 0) {
			this.client.sendKey(KeySym.ShiftL, false);
		}
		if ((arg0.stateMask & SWT.CTRL) != 0) {
			this.client.sendKey(KeySym.ControlL, false);
		}
		if ((arg0.stateMask & SWT.ALT) != 0) {
			this.client.sendKey(KeySym.AltL, false);
		}
		if ((arg0.stateMask & SWT.MOD1) != 0) {
			this.client.sendKey(KeySym.MetaL, false);
		}

	}

	public void keyReleased(KeyEvent arg0) {
		//this.client.sendKey(arg0.keyCode, false);
		// System.out.println("Sent key up command: " + arg0.keyCode);
	}

	public void mouseDoubleClick(MouseEvent arg0) {
		this.client.sendMouse(arg0.x, arg0.y, (byte) (0x01 << (arg0.button - 1)));
		this.client.sendMouse(arg0.x, arg0.y, (byte) (0));
		this.client.sendMouse(arg0.x, arg0.y, (byte) (0x01 << (arg0.button - 1)));
		this.client.sendMouse(arg0.x, arg0.y, (byte) (0));
//		System.out.println("mouseDoubleClick x: " + arg0.x + " y: " + arg0.y);
	}

	public void mouseDown(MouseEvent arg0) {
		this.client.sendMouse(arg0.x, arg0.y, (byte) (0x01 << (arg0.button - 1)));
//		System.out.println("mouseDown x: " + arg0.x + " y: " + arg0.y);
	}

	public void mouseUp(MouseEvent arg0) {
		// needed for drag 'n drop commands
		this.client.sendMouse(arg0.x, arg0.y, (byte) (0));
//		System.out.println("mouseUp x: " + arg0.x + " y: " + arg0.y);
	}

	public void focusGained(FocusEvent arg0) {
		// TODO: resume autorefresh here?
		this.getImageCanvas().setFocus();
		this.hasFocus = true;
	}

	public void focusLost(FocusEvent arg0) {
		// TODO: pause autorefresh here?
		this.hasFocus = false;
	}
	
	public boolean hasFocus() {
		return this.hasFocus;
	}

	public synchronized void setImageCanvas(ImageCanvas ic) {
		this.imageCanvas = ic;
	}

	public synchronized ImageCanvas getImageCanvas() {
		return this.imageCanvas;
	}
	
	public synchronized void setScrollCanvas(ScrollingCanvas sc) {
		this.scrollCanvas = sc;
	}
	
	public synchronized ScrollingCanvas getScrollCanvas() {
		return this.scrollCanvas;
	}

	public VNCViewer getClient() {
		return this.client;
	}
	
	
}
