/*
 * The software is provided under the Copyright, 2008.
 * Software Tools Technology Group, Computer Science Dept.,
 * Worcester Polytechnic Institute, sttg@cs.wpi.edu
 *
 * The program is provided under the terms and conditions of
 * the Eclipse Public License Version 1.0 ("EPL"). A copy of the EPL
 * is available at http://www.eclipse.org/org/documents/epl-v10.php.
 *
 * $Id:$
 */
package edu.webfoot.vnc.editor;

import gnu.rfb.server.DefaultRFBAuthenticator;
import gnu.rfb.server.RFBAuthenticator;
import gnu.rfb.server.RFBHost;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

/**
 * Coordinates between the menu bar, the VNCSessionManager and the editors.
 * 
 * @author Ken Breeman (kbreeman@wpi.edu)
 * @author Eric Griffel (sniffels@wpi.edu)
 */
public class VNCEditorActionBarContributor implements
		IWorkbenchWindowActionDelegate {

	static RFBHost host; // Only allow a single instance of the server to

	// run.

	public void run(IAction action) {
		if (action.getId().equals("start.VNC.connection")) {

			InputDialog ipd = new InputDialog(null, "Host address",
					"Enter the address of the host. ", "",
					new IInputValidator() {
						public String isValid(String newText) {
							// Accepts any number of letters or numbers
							// separated by periods, with either one colon and
							// any number [0-50], two colons and any number
							// [0-99999], or nothing afterwards
							if (newText
									.matches("((([a-zA-Z0-9]+)(\\.[a-zA-Z0-9]+)*)|(([0-9]{1,3})(\\.[0-9]{1,3}){3}))((:(([0-4]?[0-9]{1})|(50)))|(::[0-9]{1,5}))?"))
								return null;
							else
								return "Address improperly specified.";
							// (([0-9]{1,3})(\\.[0-9]{1,3}){3}) This section is
							// useless since it accepts a subset of the check it
							// is being "or"'d with. Reasonable to keep due to
							// nature of regexp check
						}
					});
			ipd.open();

			if (ipd.getReturnCode() == InputDialog.CANCEL) {
				return;
			}
			String hostLoc = ipd.getValue(); // "localhost:1";

			PasswordDialog ipd2 = new PasswordDialog(null, "Password");
			ipd2.open();

			if (ipd2.getReturnCode() == InputDialog.CANCEL) {
				return;
			}

			String password = ipd2.getValue();
			if (password == null) {
				password = "";
			}

			String viewerId = VNCSessionManager.hostToId(hostLoc);
			VNCEditor.newViewerId = viewerId; // hack :(
			try {
				VNCSessionManager.getInstance().addSession(hostLoc, password);
			} catch (Exception e3) {
				MessageDialog.openError(null, "Client initialization error", e3
						.getMessage());
				return;
			}

			try {
				Thread.sleep(10);
			} catch (InterruptedException e2) {

			}

			VNCEditor editor = null;
			try {
				editor = (VNCEditor) PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getActivePage().openEditor(
								new VNCEditorInput(), "gnu.editor.VNCViewer");
				editor.setFocus();
			} catch (PartInitException e1) {
				e1.printStackTrace();
			}

			IWorkbenchPage iwp = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage();
			// IEditorReference ier[] = iwp.getEditorReferences();
			// VNCEditor editor = null;
			// for (int c = 0; c < ier.length; c++) {
			// if (ier[c].getId().equals("gnu.editor.VNCViewer")) {
			// editor = (VNCEditor)ier[c].getEditor(false);
			// if (editor.hasFocus())
			// return;
			// }
			// }

			if (editor == null) {
				//System.out.println("ERROR: Unable to create the VNC Editor.");
				return;
			}
			// System.out.println("hasFocus: " + editor.hasFocus());
			try {

				// AutoRefresh
				editor.setRefresher(new AutoRefresh(editor));
				editor.getRefresher().restart();
				Thread arThread = new Thread(editor.getRefresher());
				arThread.start();
			} catch (Exception e) {
				e.printStackTrace();

				MessageDialog.openError(iwp.getWorkbenchWindow().getShell(),
						"Client initialization error", e.getMessage());

				iwp.closeEditor(editor, false);
			}
		} else if (action.getId().equals("stop.VNC.connection")) {
			IWorkbenchPage iwp = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage();
			IEditorReference ier[] = iwp.getEditorReferences();
			VNCEditor editor = null;
			for (int c = 0; c < ier.length; c++) {
				if (ier[c].getId().equals("gnu.editor.VNCViewer")) {
					editor = (VNCEditor) ier[c].getEditor(false);
					if (editor.hasFocus())
						break;
					else
						editor = null;
				}
			}

			if (editor != null) {
				// must be in this order
				editor.dispose();
				iwp.closeEditor(editor, false);
			}

			//System.out.println("Client stopped.");
		} else if (action.getId().equals("toggle.VNC.host")) {
			if (action.isChecked()) {
				try {
					RFBAuthenticator authenticator = new DefaultRFBAuthenticator(
							edu.webfoot.vnc.Activator
									.getDefault()
									.getPreferenceStore()
									.getString(
											edu.webfoot.vnc.PreferenceConstants.pass));
					VNCEditorActionBarContributor.host = new RFBHost(
							edu.webfoot.vnc.Activator
									.getDefault()
									.getPreferenceStore()
									.getInt(
											edu.webfoot.vnc.PreferenceConstants.port),
							edu.webfoot.vnc.Activator
									.getDefault()
									.getPreferenceStore()
									.getString(
											edu.webfoot.vnc.PreferenceConstants.hostName),
							gnu.vnc.awt.VNCRobot.class, authenticator);
				} catch (NoSuchMethodException e) {
					e.printStackTrace();
				}
			} else {
				//System.out.println("Terminating host.");
				VNCEditorActionBarContributor.host.close();
			}
		} else {
			//System.out.println("Unable to recognize button.");
		}
	}

	public void dispose() {
	}

	public void init(IWorkbenchWindow window) {
	}

	public void selectionChanged(IAction action, ISelection selection) {
	}

}
