/*
 * The software is provided under the Copyright, 2008.
 * Software Tools Technology Group, Computer Science Dept.,
 * Worcester Polytechnic Institute, sttg@cs.wpi.edu
 *
 * The program is provided under the terms and conditions of
 * the Eclipse Public License Version 1.0 ("EPL"). A copy of the EPL
 * is available at http://www.eclipse.org/org/documents/epl-v10.php.
 *
 * $Id:$
 */
package edu.webfoot.vnc.editor;

import java.util.ArrayList;

import gnu.vnc.VNCViewer;

/**
 * Manages sessions of VNCClients.
 * This keeps track of all of the sessions in one logical place.
 * Prevents multiple sessions from being spawned to the same host::port.
 * 
 * @author Ken  Breeman (kbreeman@wpi.edu)
 * @author Eric Griffel (sniffels@wpi.edu)
 */
public class VNCSessionManager {

	private static VNCSessionManager instance;
	private ArrayList<VNCViewer> sessions;

	/**
	 * This is a singleton. There can only be one. Manages all of the VNC
	 * sessions, each session is identified by host:port.
	 * 
	 * @return The singleton instance.
	 */
	public synchronized static VNCSessionManager getInstance() {
		if (instance == null) {
			instance = new VNCSessionManager();
		}
		return instance;
	}
	
	public static String hostToId(String host) {
		if(host.contains("::")) {
			return host;
		} else if(host.contains(":")) {
			return host.split(":")[0] + (Integer.parseInt(host.split(":")[1]) + 5900);
		} else {
			return (host + "::5900");
		}
	}

	private VNCSessionManager() {
		sessions = new ArrayList<VNCViewer>();
	}

	public synchronized VNCViewer getSession(String name) {
		for (int i = 0; i < sessions.size(); i++) {
			if (sessions.get(i).getServerHost().equalsIgnoreCase(name)) {
				return sessions.get(i);
			}
		}

		return null;
	}

	public synchronized void addSession(String hostport, String pass)
			throws Exception {
		// Prune the existing sessions.
		for (int i = 0; i < sessions.size(); i++) {
			if (sessions.get(i).getServerHost().equalsIgnoreCase(hostport)) {
				sessions.get(i).stop();
				sessions.remove(i);
			}
		}

		// Create a temp variable in case of exceptions.
		VNCViewer temp = new VNCViewer(hostport, pass);

		// Add it to the list.
		sessions.add(temp);
	}

	public synchronized void killSession(String hostport) {
		for (int i = 0; i < sessions.size(); i++) {
			if (sessions.get(i).getServerHost().equalsIgnoreCase(hostport)) {
				sessions.get(i).stop();
				sessions.remove(i);
			}
		}
	}

	public synchronized void killAllSessions() {
		for (int i = 0; i < sessions.size(); i++) {
			sessions.get(i).stop();
			sessions.remove(i);
		}
	}
}
