package gnu.rfb;

/**
 * <p>
 * <br><br><center><table border="1" width="80%"><hr>
 * <strong><a href="http://www.amherst.edu/~tliron/vncj">VNCj</a></strong>
 * <p>
 * Copyright (C) 2000-2002 by Tal Liron
 * <p>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * <p>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * <a href="http://www.gnu.org/copyleft/lesser.html">GNU Lesser General Public License</a>
 * for more details.
 * <p>
 * You should have received a copy of the <a href="http://www.gnu.org/copyleft/lesser.html">
 * GNU Lesser General Public License</a> along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * <hr></table></center>
 **/

import java.io.DataOutput;
import java.io.IOException;
import java.util.Vector;

/**
 * CoRRE - Compact RRE Encoding.
 * <p>
 * We have an RRE header structure giving the number of subrectangles following.
 * Finally the data follows in the form [<bgpixel><subrect><subrect>...]
 * where each <subrect> is [<pixel><rfbCoRRERectangle>]. This means that the
 * whole rectangle must be at most 255x255 pixels.
 */
public class CoRREStub extends Rect {

	//
	public CoRRE rects[];

	/**
	 * Constructor to encode an array of pixels into a CoRRE,
	 * 
	 * @param pixels
	 *            The array of pixels to encode.
	 * @param pixelFormat
	 *            The current pixel format of the image.
	 * @param offsetX
	 *            The offset in the X direction.
	 * @param offsetY
	 *            The offset in the Y direction.
	 * @param scanline
	 *            The width of a single scan line.
	 * @param x
	 *            The X-coordinate of the image.
	 * @param y
	 *            The Y-coordinate of the image.
	 * @param w
	 *            The width of the image.
	 * @param h
	 *            The height of the image
	 */
	public CoRREStub(int[] pixels, PixelFormat pixelFormat, int offsetX,
			int offsetY, int scanline, int x, int y, int w, int h) {
		super(x, y, w, h);

		CoRRE rect;
		Vector<CoRRE> vector = new Vector<CoRRE>();

		if ((w <= 0xFF) && (h <= 0xFF)) {
			rect = new CoRRE(pixels, pixelFormat, offsetX, offsetY, scanline,
					x, y, w, h);
			vector.addElement(rect);
		} else {
			int currentX, currentY, currentW, currentH;
			for (currentY = 0; currentY < h; currentY += 0xFF) {
				for (currentX = 0; currentX < w; currentX += 0xFF) {
					currentW = w - currentX;
					currentH = h - currentY;

					if (currentW > 0xFF)
						currentW = 0xFF;
					if (currentH > 0xFF)
						currentH = 0xFF;

					rect = new CoRRE(pixels, pixelFormat, offsetX, offsetY,
							scanline, x + currentX, y + currentY, currentW,
							currentH);
					vector.addElement(rect);
				}
			}
		}

		rects = new CoRRE[vector.size()];
		vector.toArray((Object[]) rects);
		count = rects.length;
	}

	/**
	 * Constructor for encoding an array of CoRRE rectangles.
	 * 
	 * @param x
	 *            The X-coordinate.
	 * @param y
	 *            The y-coordinate.
	 * @param w
	 *            The width.
	 * @param h
	 *            The height.
	 * @param rects
	 *            The rectangles to encode.
	 */
	public CoRREStub(int x, int y, int w, int h, CoRRE[] rects) {
		super(x, y, w, h);
		this.rects = rects;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gnu.rfb.Rect#writeData(java.io.DataOutput)
	 */
	public void writeData(DataOutput output) throws IOException {
		for (int i = 0; i < rects.length; i++)
			rects[i].writeData(output);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gnu.rfb.Rect#transform(int, int)
	 */
	public void transform(int transformX, int transformY) {
		for (int i = 0; i < rects.length; i++)
			rects[i].transform(transformX, transformY);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gnu.rfb.Rect#clone()
	 */
	public Object clone() throws CloneNotSupportedException {
		CoRRE[] rectsClone = new CoRRE[rects.length];
		for (int i = 0; i < rects.length; i++)
			rectsClone[i] = (CoRRE) rects[i].clone();

		return new CoRREStub(x, y, w, h, rectsClone);
	}
}
