package gnu.rfb;

/**
 * <br><br><center><table border="1" width="80%"><hr>
 * <strong><a href="http://www.amherst.edu/~tliron/vncj">VNCj</a></strong>
 * <p>
 * Copyright (C) 2000-2002 by Tal Liron
 * <p>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * <p>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * <a href="http://www.gnu.org/copyleft/lesser.html">GNU Lesser General Public License</a>
 * for more details.
 * <p>
 * You should have received a copy of the <a href="http://www.gnu.org/copyleft/lesser.html">
 * GNU Lesser General Public License</a> along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * <hr></table></center>
 **/

import java.awt.event.KeyEvent;
import java.util.ArrayList;

/**
 * Key Symbol table class.
 * 
 * Modified heavily by:
 *  Ken  Breeman (kbreeman@wpi.edu)
 *  Eric Griffel (sniffels@wpi.edu)
 */
public abstract class KeySym {
	public static final int DeadGrave = 0xFE50;
	public static final int DeadAcute = 0xFE51;
	public static final int DeadCircumflex = 0xFE52;
	public static final int DeadTilde = 0xFE53;

	public static final int BackSpace = 0xFF08;
	public static final int Tab = 0xFF09;
	public static final int Linefeed = 0xFF0A;
	public static final int Clear = 0xFF0B;
	public static final int Return = 0xFF0D;
	public static final int Pause = 0xFF13;
	public static final int ScrollLock = 0xFF14;
	public static final int SysReq = 0xFF15;
	public static final int Escape = 0xFF1B;

	public static final int Delete = 0xFFFF;

	public static final int Home = 0xFF50;
	public static final int Left = 0xFF51;
	public static final int Up = 0xFF52;
	public static final int Right = 0xFF53;
	public static final int Down = 0xFF54;
	public static final int PageUp = 0xFF55;
	public static final int PageDown = 0xFF56;
	public static final int End = 0xFF57;
	public static final int Begin = 0xFF58;

	public static final int Select = 0xFF60;
	public static final int Print = 0xFF61;
	public static final int Execute = 0xFF62;
	public static final int Insert = 0xFF63;

	public static final int Cancel = 0xFF69;
	public static final int Help = 0xFF6A;
	public static final int Break = 0xFF6B;
	public static final int NumLock = 0xFF6F;

	public static final int KpSpace = 0xFF80;
	public static final int KpTab = 0xFF89;
	public static final int KpEnter = 0xFF8D;

	public static final int KpHome = 0xFF95;
	public static final int KpLeft = 0xFF96;
	public static final int KpUp = 0xFF97;
	public static final int KpRight = 0xFF98;
	public static final int KpDown = 0xFF99;
	public static final int KpPrior = 0xFF9A;
	public static final int KpPageUp = 0xFF9A;
	public static final int KpNext = 0xFF9B;
	public static final int KpPageDown = 0xFF9B;
	public static final int KpEnd = 0xFF9C;
	public static final int KpBegin = 0xFF9D;
	public static final int KpInsert = 0xFF9E;
	public static final int KpDelete = 0xFF9F;
	public static final int KpEqual = 0xFFBD;
	public static final int KpMultiply = 0xFFAA;
	public static final int KpAdd = 0xFFAB;
	public static final int KpSeparator = 0xFFAC;
	public static final int KpSubtract = 0xFFAD;
	public static final int KpDecimal = 0xFFAE;
	public static final int KpDivide = 0xFFAF;

	public static final int KpF1 = 0xFF91;
	public static final int KpF2 = 0xFF92;
	public static final int KpF3 = 0xFF93;
	public static final int KpF4 = 0xFF94;

	public static final int Kp0 = 0xFFB0;
	public static final int Kp1 = 0xFFB1;
	public static final int Kp2 = 0xFFB2;
	public static final int Kp3 = 0xFFB3;
	public static final int Kp4 = 0xFFB4;
	public static final int Kp5 = 0xFFB5;
	public static final int Kp6 = 0xFFB6;
	public static final int Kp7 = 0xFFB7;
	public static final int Kp8 = 0xFFB8;
	public static final int Kp9 = 0xFFB9;

	public static final int F1 = 0xFFBE;
	public static final int F2 = 0xFFBF;
	public static final int F3 = 0xFFC0;
	public static final int F4 = 0xFFC1;
	public static final int F5 = 0xFFC2;
	public static final int F6 = 0xFFC3;
	public static final int F7 = 0xFFC4;
	public static final int F8 = 0xFFC5;
	public static final int F9 = 0xFFC6;
	public static final int F10 = 0xFFC7;
	public static final int F11 = 0xFFC8;
	public static final int F12 = 0xFFC9;
	public static final int F13 = 0xFFCA;
	public static final int F14 = 0xFFCB;
	public static final int F15 = 0xFFCC;
	public static final int F16 = 0xFFCD;
	public static final int F17 = 0xFFCE;
	public static final int F18 = 0xFFCF;
	public static final int F19 = 0xFFD0;
	public static final int F20 = 0xFFD1;
	public static final int F21 = 0xFFD2;
	public static final int F22 = 0xFFD3;
	public static final int F23 = 0xFFD4;
	public static final int F24 = 0xFFD5;
	
	public static final int ASCII_SPACE				= 0x0020;
	public static final int ASCII_EXCLAMATION_MARK	= 0x0021;
	public static final int ASCII_DOUBLE_QUOTE		= 0x0022;
	public static final int ASCII_NUMBER_SIGN		= 0x0023;
	public static final int ASCII_DOLLAR_SIGN		= 0x0024;
	public static final int ASCII_PERCENT_SIGN		= 0x0025;
	public static final int ASCII_AMPERSAND			= 0x0026;
	public static final int ASCII_QUOTE				= 0x0027;
	public static final int ASCII_LEFT_PAREN		= 0x0028;
	public static final int ASCII_RIGHT_PAREN		= 0x0029;
	public static final int ASCII_ASTERISK			= 0x002A;
	public static final int ASCII_PLUS				= 0x002B;
	public static final int ASCII_COMMA				= 0x002C;
	public static final int ASCII_MINUS				= 0x002D;
	public static final int ASCII_PERIOD			= 0x002E;
	public static final int ASCII_SLASH				= 0x002F;

	public static final int ASCII_0 = 0x0030;
	public static final int ASCII_1 = 0x0031;
	public static final int ASCII_2 = 0x0032;
	public static final int ASCII_3 = 0x0033;
	public static final int ASCII_4 = 0x0034;
	public static final int ASCII_5 = 0x0035;
	public static final int ASCII_6 = 0x0036;
	public static final int ASCII_7 = 0x0037;
	public static final int ASCII_8 = 0x0038;
	public static final int ASCII_9 = 0x0039;

	public static final int ASCII_COLON			= 0x003A;
	public static final int ASCII_SEMICOLON		= 0x003B;
	public static final int ASCII_LESSTHAN		= 0x003C;
	public static final int ASCII_EQUALS		= 0x003D;
	public static final int ASCII_GREATERTHAN	= 0x003E;
	public static final int ASCII_QUESTION_MARK	= 0x003F;
	public static final int ASCII_AT			= 0x0040;
	
	public static final int ASCII_A = 0x0041;
	public static final int ASCII_B = 0x0042;
	public static final int ASCII_C = 0x0043;
	public static final int ASCII_D = 0x0044;
	public static final int ASCII_E = 0x0045;
	public static final int ASCII_F = 0x0046;
	public static final int ASCII_G = 0x0047;
	public static final int ASCII_H = 0x0048;
	public static final int ASCII_I = 0x0049;
	public static final int ASCII_J = 0x004A;
	public static final int ASCII_K = 0x004B;
	public static final int ASCII_L = 0x004C;
	public static final int ASCII_M = 0x004D;
	public static final int ASCII_N = 0x004E;
	public static final int ASCII_O = 0x004F;
	public static final int ASCII_P = 0x0050;
	public static final int ASCII_Q = 0x0051;
	public static final int ASCII_R = 0x0052;
	public static final int ASCII_S = 0x0053;
	public static final int ASCII_T = 0x0054;
	public static final int ASCII_U = 0x0055;
	public static final int ASCII_V = 0x0056;
	public static final int ASCII_W = 0x0057;
	public static final int ASCII_X = 0x0058;
	public static final int ASCII_Y = 0x0059;
	public static final int ASCII_Z = 0x005A;
	
	public static final int ASCII_OPEN_BRACKET	= 0x005B;
	public static final int ASCII_BACK_SLASH	= 0x005C;
	public static final int ASCII_CLOSE_BRACKET	= 0x005D;
	public static final int ASCII_CIRCUMFLEX	= 0x005E;
	public static final int ASCII_UNDER_SCORE	= 0x005F;
	public static final int ASCII_BACK_QUOTE	= 0x0060;

	public static final int ASCII_a = 0x0061;
	public static final int ASCII_b = 0x0062;
	public static final int ASCII_c = 0x0063;
	public static final int ASCII_d = 0x0064;
	public static final int ASCII_e = 0x0065;
	public static final int ASCII_f = 0x0066;
	public static final int ASCII_g = 0x0067;
	public static final int ASCII_h = 0x0068;
	public static final int ASCII_i = 0x0069;
	public static final int ASCII_j = 0x006A;
	public static final int ASCII_k = 0x006B;
	public static final int ASCII_l = 0x006C;
	public static final int ASCII_m = 0x006D;
	public static final int ASCII_n = 0x006E;
	public static final int ASCII_o = 0x006F;
	public static final int ASCII_p = 0x0070;
	public static final int ASCII_q = 0x0071;
	public static final int ASCII_r = 0x0072;
	public static final int ASCII_s = 0x0073;
	public static final int ASCII_t = 0x0074;
	public static final int ASCII_u = 0x0075;
	public static final int ASCII_v = 0x0076;
	public static final int ASCII_w = 0x0077;
	public static final int ASCII_x = 0x0078;
	public static final int ASCII_y = 0x0079;
	public static final int ASCII_z = 0x007A;
	
	public static final int ASCII_LEFT_BRACE	= 0x007B;
	public static final int ASCII_PIPE			= 0x007C;
	public static final int ASCII_RIGHT_BRACE	= 0x007D;
	public static final int ASCII_TILDE			= 0x007E;
	
	public static final int ShiftL = 0xFFE1;
	public static final int ShiftR = 0xFFE2;
	public static final int ControlL = 0xFFE3;
	public static final int ControlR = 0xFFE4;
	public static final int CapsLock = 0xFFE5;
	public static final int ShiftLock = 0xFFE6;
	public static final int MetaL = 0xFFE7;
	public static final int MetaR = 0xFFE8;
	public static final int AltL = 0xFFE9;
	public static final int AltR = 0xFFEA;

	/**
	 * Converts from a VNC KeySym value to a Java KeyEvent value.
	 * 
	 * @param keysym
	 *            The VNC KeySym value.
	 * @return The integer KeyEvent value or -1 if the key cannot be generated
	 *         by a single Java KeyEvent.
	 */
	public static int toVK(int keysym) {
		switch (keysym) {
		
		case ASCII_SPACE:
			return KeyEvent.VK_SPACE;
		case ASCII_EXCLAMATION_MARK:
			//return KeyEvent.VK_EXCLAMATION_MARK; //EXPLODES
			return -1;
		case ASCII_DOUBLE_QUOTE:
			//return KeyEvent.VK_QUOTEDBL; //EXPLODES
			return -1;
		case ASCII_NUMBER_SIGN:
			//return KeyEvent.VK_NUMBER_SIGN; //EXPLODES
			return -1;
		case ASCII_DOLLAR_SIGN:
			//return KeyEvent.VK_DOLLAR; //EXPLODES
			return -1;
		case ASCII_PERCENT_SIGN:
			//return KeyEvent.??????; // DNE
			return -1;
		case ASCII_AMPERSAND:
			//return KeyEvent.VK_AMPERSAND; //EXPLODES
			return -1;
		case ASCII_QUOTE:
			return KeyEvent.VK_QUOTE;
		case ASCII_LEFT_PAREN:
			//return KeyEvent.VK_LEFT_PARENTHESIS; //EXPLODES
			return -1;
		case ASCII_RIGHT_PAREN:
			//return KeyEvent.VK_RIGHT_PARENTHESIS; //EXPLODES
			return -1;
		case ASCII_ASTERISK:
			//return KeyEvent.VK_ASTERISK; //EXPLODES
			return -1;
		case ASCII_PLUS:
			return KeyEvent.VK_ADD;
		case ASCII_COMMA:
			return KeyEvent.VK_COMMA;
		case ASCII_MINUS:
			return KeyEvent.VK_MINUS;
		case ASCII_PERIOD:
			return KeyEvent.VK_PERIOD;
		case ASCII_SLASH:
			return KeyEvent.VK_SLASH;
		case ASCII_0:
			return KeyEvent.VK_0;
		case ASCII_1:
			return KeyEvent.VK_1;
		case ASCII_2:
			return KeyEvent.VK_2;
		case ASCII_3:
			return KeyEvent.VK_3;
		case ASCII_4:
			return KeyEvent.VK_4;
		case ASCII_5:
			return KeyEvent.VK_5;
		case ASCII_6:
			return KeyEvent.VK_6;
		case ASCII_7:
			return KeyEvent.VK_7;
		case ASCII_8:
			return KeyEvent.VK_8;
		case ASCII_9:
			return KeyEvent.VK_9;

		case ASCII_COLON:
			//return KeyEvent.VK_COLON; //EXPLODES
			return -1;
		case ASCII_SEMICOLON:
			return KeyEvent.VK_SEMICOLON;
		case ASCII_LESSTHAN:
			//return KeyEvent.VK_LESS; //EXPLODES
			return -1;
		case ASCII_EQUALS:
			return KeyEvent.VK_EQUALS;
		case ASCII_GREATERTHAN:
			//return KeyEvent.VK_GREATER; //EXPLODES
			return -1;
		case ASCII_QUESTION_MARK:
			//return KeyEvent.VK_PROPS; //EXPLODES
			return -1;
		case ASCII_AT:
			//return KeyEvent.VK_AT; //EXPLODES
			return -1;
		
		case ASCII_a:
			return KeyEvent.VK_A;
		case ASCII_b:
			return KeyEvent.VK_B;
		case ASCII_c:
			return KeyEvent.VK_C;
		case ASCII_d:
			return KeyEvent.VK_D;
		case ASCII_e:
			return KeyEvent.VK_E;
		case ASCII_f:
			return KeyEvent.VK_F;
		case ASCII_g:
			return KeyEvent.VK_G;
		case ASCII_h:
			return KeyEvent.VK_H;
		case ASCII_i:
			return KeyEvent.VK_I;
		case ASCII_j:
			return KeyEvent.VK_J;
		case ASCII_k:
			return KeyEvent.VK_K;
		case ASCII_l:
			return KeyEvent.VK_L;
		case ASCII_m:
			return KeyEvent.VK_M;
		case ASCII_n:
			return KeyEvent.VK_N;
		case ASCII_o:
			return KeyEvent.VK_O;
		case ASCII_p:
			return KeyEvent.VK_P;
		case ASCII_q:
			return KeyEvent.VK_Q;
		case ASCII_r:
			return KeyEvent.VK_R;
		case ASCII_s:
			return KeyEvent.VK_S;
		case ASCII_t:
			return KeyEvent.VK_T;
		case ASCII_u:
			return KeyEvent.VK_U;
		case ASCII_v:
			return KeyEvent.VK_V;
		case ASCII_w:
			return KeyEvent.VK_W;
		case ASCII_x:
			return KeyEvent.VK_X;
		case ASCII_y:
			return KeyEvent.VK_Y;
		case ASCII_z:
			return KeyEvent.VK_Z;
		
		case ASCII_OPEN_BRACKET:
			return KeyEvent.VK_OPEN_BRACKET;
		case ASCII_BACK_SLASH:
			return KeyEvent.VK_BACK_SLASH;
		case ASCII_CLOSE_BRACKET:
			return KeyEvent.VK_CLOSE_BRACKET;
		case ASCII_CIRCUMFLEX:
			//return KeyEvent.VK_CIRCUMFLEX; // EXPLODES
			return -1;
		case ASCII_UNDER_SCORE:
			//return KeyEvent.VK_UNDERSCORE; // EXPLODES
			return -1;
		case ASCII_BACK_QUOTE:
			return KeyEvent.VK_BACK_QUOTE;
			
		case ASCII_A:
		case ASCII_B:
		case ASCII_C:
		case ASCII_D:
		case ASCII_E:
		case ASCII_F:
		case ASCII_G:
		case ASCII_H:
		case ASCII_I:
		case ASCII_J:
		case ASCII_K:
		case ASCII_L:
		case ASCII_M:
		case ASCII_N:
		case ASCII_O:
		case ASCII_P:
		case ASCII_Q:
		case ASCII_R:
		case ASCII_S:
		case ASCII_T:
		case ASCII_U:
		case ASCII_V:
		case ASCII_W:
		case ASCII_X:
		case ASCII_Y:
		case ASCII_Z:
			return -1; //Return -1 and use the combo key for capital letters.
		
		case ASCII_LEFT_BRACE:
			//return KeyEvent.VK_BRACELEFT;
			return -1;
		case ASCII_PIPE:
			//return KeyEvent.VK_SEPARATOR;
			return -1;
		case ASCII_RIGHT_BRACE:
			//return KeyEvent.VK_BRACERIGHT;
			return -1;
		case ASCII_TILDE:
			//return KeyEvent.VK_DEAD_TILDE;
			return -1;



		case DeadGrave:
			return KeyEvent.VK_DEAD_GRAVE;
		case DeadAcute:
			return KeyEvent.VK_DEAD_ACUTE;
		case DeadCircumflex:
			return KeyEvent.VK_DEAD_CIRCUMFLEX;
		case DeadTilde:
			return KeyEvent.VK_DEAD_TILDE;

		case BackSpace:
			return KeyEvent.VK_BACK_SPACE;
		case Tab:
			return KeyEvent.VK_TAB;
			// No Java equivalent: case Linefeed: return KeyEvent.;
		case Clear:
			return KeyEvent.VK_CLEAR;
		case Return:
			return KeyEvent.VK_ENTER;
		case Pause:
			return KeyEvent.VK_PAUSE;
		case ScrollLock:
			return KeyEvent.VK_SCROLL_LOCK;
			// No Java equivalent: case SysReq: return KeyEvent.;
		case Escape:
			return KeyEvent.VK_ESCAPE;

		case Delete:
			return KeyEvent.VK_DELETE;

		case Home:
			return KeyEvent.VK_HOME;
		case Left:
			return KeyEvent.VK_LEFT;
		case Up:
			return KeyEvent.VK_UP;
		case Right:
			return KeyEvent.VK_RIGHT;
		case Down:
			return KeyEvent.VK_DOWN;
		case PageUp:
			return KeyEvent.VK_PAGE_UP;
		case PageDown:
			return KeyEvent.VK_PAGE_DOWN;
		case End:
			return KeyEvent.VK_END;
			// No Java equivalent: case Begin: return KeyEvent.;

			// No Java equivalent: case Select: return KeyEvent.;
		case Print:
			return KeyEvent.VK_PRINTSCREEN;
			// No Java equivalent: case Execute: return KeyEvent.;
		case Insert:
			return KeyEvent.VK_INSERT;

		case Cancel:
			return KeyEvent.VK_CANCEL;
		case Help:
			return KeyEvent.VK_HELP;
			// No Java equivalent: case Break: return KeyEvent.;
		case NumLock:
			return KeyEvent.VK_NUM_LOCK;

		case KpSpace:
			return KeyEvent.VK_SPACE;
		case KpTab:
			return KeyEvent.VK_TAB;
		case KpEnter:
			return KeyEvent.VK_ENTER;

		case KpHome:
			return KeyEvent.VK_HOME;
		case KpLeft:
			return KeyEvent.VK_LEFT;
		case KpUp:
			return KeyEvent.VK_UP;
		case KpRight:
			return KeyEvent.VK_RIGHT;
		case KpDown:
			return KeyEvent.VK_DOWN;
		case KpPageUp:
			return KeyEvent.VK_PAGE_UP; // = KpPrior
		case KpPageDown:
			return KeyEvent.VK_PAGE_DOWN; // = KpNext
		case KpEnd:
			return KeyEvent.VK_END;
			// No Java equivalent: case KpBegin: return KeyEvent.;
		case KpInsert:
			return KeyEvent.VK_INSERT;
		case KpDelete:
			return KeyEvent.VK_DELETE;
		case KpEqual:
			return KeyEvent.VK_EQUALS;
		case KpMultiply:
			return KeyEvent.VK_MULTIPLY;
		case KpAdd:
			return KeyEvent.VK_ADD;
		case KpSeparator:
			return KeyEvent.VK_SEPARATER; // Sun should spellcheck...
		case KpSubtract:
			return KeyEvent.VK_SUBTRACT;
		case KpDecimal:
			return KeyEvent.VK_DECIMAL;
		case KpDivide:
			return KeyEvent.VK_DIVIDE;

		case KpF1:
			return KeyEvent.VK_F1;
		case KpF2:
			return KeyEvent.VK_F2;
		case KpF3:
			return KeyEvent.VK_F3;
		case KpF4:
			return KeyEvent.VK_F4;

		case Kp0:
			return KeyEvent.VK_NUMPAD0;
		case Kp1:
			return KeyEvent.VK_NUMPAD1;
		case Kp2:
			return KeyEvent.VK_NUMPAD2;
		case Kp3:
			return KeyEvent.VK_NUMPAD3;
		case Kp4:
			return KeyEvent.VK_NUMPAD4;
		case Kp5:
			return KeyEvent.VK_NUMPAD5;
		case Kp6:
			return KeyEvent.VK_NUMPAD6;
		case Kp7:
			return KeyEvent.VK_NUMPAD7;
		case Kp8:
			return KeyEvent.VK_NUMPAD8;
		case Kp9:
			return KeyEvent.VK_NUMPAD9;

		case F1:
			return KeyEvent.VK_F1;
		case F2:
			return KeyEvent.VK_F2;
		case F3:
			return KeyEvent.VK_F3;
		case F4:
			return KeyEvent.VK_F4;
		case F5:
			return KeyEvent.VK_F5;
		case F6:
			return KeyEvent.VK_F6;
		case F7:
			return KeyEvent.VK_F7;
		case F8:
			return KeyEvent.VK_F8;
		case F9:
			return KeyEvent.VK_F9;
		case F10:
			return KeyEvent.VK_F10;
		case F11:
			return KeyEvent.VK_F11;
		case F12:
			return KeyEvent.VK_F12;
		case F13:
			return KeyEvent.VK_F12;
		case F14:
			return KeyEvent.VK_F12;
		case F15:
			return KeyEvent.VK_F12;
		case F16:
			return KeyEvent.VK_F12;
		case F17:
			return KeyEvent.VK_F12;
		case F18:
			return KeyEvent.VK_F12;
		case F19:
			return KeyEvent.VK_F12;
		case F20:
			return KeyEvent.VK_F12;
		case F21:
			return KeyEvent.VK_F12;
		case F22:
			return KeyEvent.VK_F12;
		case F23:
			return KeyEvent.VK_F12;
		case F24:
			return KeyEvent.VK_F12;

		case CapsLock:
			return KeyEvent.VK_CAPS_LOCK;
			// No Java equivalent: case ShiftLock: return KeyEvent.;
		default:
			return 0;
		}
	}

	/**
	 * Converts from a VNC KeySym value to a Java KeyEvent value. 
	 * Includes ctrl, alt, shift, and meta keys.
	 * 
	 * @param keysym
	 *            The VNC KeySym value.
	 * @return The Java KeyEvent value.
	 */
	public static int toVKall(int keysym) {
		
		//Check for non meta keys
		int key = toVK(keysym);
		if (key != 0) {
			return key;
		}
		
			

		switch (keysym) {
		case ShiftL:
			return KeyEvent.VK_SHIFT;
		case ShiftR:
			return KeyEvent.VK_SHIFT;
		case ControlL:
			return KeyEvent.VK_CONTROL;
		case ControlR:
			return KeyEvent.VK_CONTROL;
		case MetaL:
			return KeyEvent.VK_META;
		case MetaR:
			return KeyEvent.VK_META;
		case AltL:
			return KeyEvent.VK_ALT;
		case AltR:
			return KeyEvent.VK_ALT;
		default:
			return 0;
		}
	}

	/**
	 * Converts from a VNC KeySym value to a Java KeyEvent mask value.
	 * 
	 * @param keysym The VNC KeySym value.
	 * @return The Java KeyEvent mask value.
	 */
	public static int toMask(int keysym) {
		switch (keysym) {
		case ShiftL:
			return KeyEvent.SHIFT_MASK;
		case ShiftR:
			return KeyEvent.SHIFT_MASK;
		case ControlL:
			return KeyEvent.CTRL_MASK;
		case ControlR:
			return KeyEvent.CTRL_MASK;
		case MetaL:
			return KeyEvent.META_MASK;
		case MetaR:
			return KeyEvent.META_MASK;
		case AltL:
			return KeyEvent.ALT_MASK;
		case AltR:
			return KeyEvent.ALT_MASK;
		default:
			return 0;
		}
	}
	
	/**
	 * Converts from a VNC KeySym value to a series of Java KeyEvent values.
	 * Enables support for capital letters and other combinations of keys.
	 * 
	 * @param key
	 *            The VNC KeySym value.
	 * @return An ArrayList of Java KeyEvent values.
	 * 
	 * @author Ken  Breeman (kbreeman@wpi.edu)
	 * @author Eric Griffel (sniffels@wpi.edu)
	 */
	public static ArrayList<Integer> VKcombo(int key) {
		ArrayList<Integer> keys = new ArrayList<Integer>();
		
		switch (key) {
			case ASCII_TILDE:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_BACK_QUOTE);
				break;
			case ASCII_EXCLAMATION_MARK:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_1);
				break;
			case ASCII_AT:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_2);
				break;
			case ASCII_NUMBER_SIGN:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_3);
				break;
			case ASCII_DOLLAR_SIGN:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_4);
				break;
			case ASCII_PERCENT_SIGN:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_5);
				break;
			case ASCII_CIRCUMFLEX:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_6);
				break;
			case ASCII_AMPERSAND:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_7);
				break;
			case ASCII_ASTERISK:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_8);
				break;
			case ASCII_LEFT_PAREN:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_9);
				break;
			case ASCII_RIGHT_PAREN:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_0);
				break;
			case ASCII_UNDER_SCORE:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_MINUS);
				break;
			case ASCII_PLUS:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_EQUALS);
				break;
			case ASCII_LEFT_BRACE:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_OPEN_BRACKET);
				break;
			case ASCII_RIGHT_BRACE:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_CLOSE_BRACKET);
				break;
			case ASCII_PIPE:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_BACK_SLASH);
				break;
			case ASCII_COLON:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_SEMICOLON);
				break;
			case ASCII_DOUBLE_QUOTE:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_QUOTE);
				break;
			case ASCII_LESSTHAN:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_COMMA);
				break;
			case ASCII_GREATERTHAN:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_PERIOD);
				break;
			case ASCII_QUESTION_MARK:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_SLASH);
				break;
			
			case ASCII_A:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_A);
				break;
			case ASCII_B:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_B);
				break;
			case ASCII_C:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_C);
				break;
			case ASCII_D:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_D);
				break;
			case ASCII_E:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_E);
				break;
			case ASCII_F:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_F);
				break;
			case ASCII_G:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_G);
				break;
			case ASCII_H:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_H);
				break;
			case ASCII_I:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_I);
				break;
			case ASCII_J:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_J);
				break;
			case ASCII_K:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_K);
				break;
			case ASCII_L:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_L);
				break;
			case ASCII_M:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_M);
				break;
			case ASCII_N:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_N);
				break;
			case ASCII_O:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_O);
				break;
			case ASCII_P:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_P);
				break;
			case ASCII_Q:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_Q);
				break;
			case ASCII_R:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_R);
				break;
			case ASCII_S:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_S);
				break;
			case ASCII_T:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_T);
				break;
			case ASCII_U:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_U);
				break;
			case ASCII_V:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_V);
				break;
			case ASCII_W:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_W);
				break;
			case ASCII_X:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_X);
				break;
			case ASCII_Y:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_Y);
				break;
			case ASCII_Z:
				keys.add(KeyEvent.VK_SHIFT);
				keys.add(KeyEvent.VK_Z);
				break;
			
		}
		return keys;
	}
}
