package gnu.rfb;

/**
 * <br><br><center><table border="1" width="80%"><hr>
 * <strong><a href="http://www.amherst.edu/~tliron/vncj">VNCj</a></strong>
 * <p>
 * Copyright (C) 2000-2002 by Tal Liron
 * <p>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * <p>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * <a href="http://www.gnu.org/copyleft/lesser.html">GNU Lesser General Public License</a>
 * for more details.
 * <p>
 * You should have received a copy of the <a href="http://www.gnu.org/copyleft/lesser.html">
 * GNU Lesser General Public License</a> along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * <hr></table></center>
 **/

import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.IOException;

/**
 * Provides encoding and decoding of images using the Raw encoding.
 */
public class Raw extends Rect {

	// The current PixelFormat to use.
	public PixelFormat pixelFormat;

	// Temporary storage for the un-decoded pixels.
	public byte[] bytes;

	// Temporary storage for the un-encoded pixels.
	public int[] pixels;

	// Input for decoding directly from the socket.
	private DataInputStream input;

	/**
	 * Constructor for encoding an image into Raw.
	 * 
	 * @param pixels
	 * @param pixelFormat
	 * @param offsetX
	 * @param offsetY
	 * @param scanline
	 * @param x
	 * @param y
	 * @param w
	 * @param h
	 */
	public Raw(int[] pixels, PixelFormat pixelFormat, int offsetX, int offsetY,
			int scanline, int x, int y, int w, int h) {
		super(x, y, w, h);
		this.pixelFormat = pixelFormat;

		// Encode as bytes
		int b = 0;
		int i = 0;
		int s = 0;
		int pixel;
		int size = w * h;
		int jump = scanline - w;
		int p = (y - offsetY) * scanline + x - offsetX;
		switch (pixelFormat.bitsPerPixel) {
		case 32:
			bytes = new byte[size << 2];
			for (; i < size; i++, s++, p++) {
				if (s == w) {
					s = 0;
					p += jump;
				}
				pixel = pixelFormat.translatePixel(pixels[p]);
				bytes[b++] = (byte) (pixel & 0xFF);
				bytes[b++] = (byte) ((pixel >> 8) & 0xFF);
				bytes[b++] = (byte) ((pixel >> 16) & 0xFF);
				bytes[b++] = (byte) ((pixel >> 24) & 0xFF);
			}
			break;
		case 16:
			bytes = new byte[size << 1];
			for (; i < size; i++, s++, p++) {
				if (s == w) {
					s = 0;
					p += jump;
				}
				pixel = pixelFormat.translatePixel(pixels[p]);
				bytes[b++] = (byte) (pixel & 0xFF);
				bytes[b++] = (byte) ((pixel >> 8) & 0xFF);
			}
			break;
		case 8:
			bytes = new byte[size];
			for (; i < size; i++, s++, p++) {
				if (s == w) {
					s = 0;
					p += jump;
				}
				bytes[i] = (byte) pixelFormat.translatePixel(pixels[p]);
			}
			break;
		}
	}

	/**
	 * Constructor for encoding an image into Raw.
	 * 
	 * @param x
	 * @param y
	 * @param w
	 * @param h
	 * @param pixelFormat
	 * @param bytes
	 */
	public Raw(int x, int y, int w, int h, PixelFormat pixelFormat, byte[] bytes) {
		super(x, y, w, h);
		this.pixelFormat = pixelFormat;
		this.bytes = bytes;
	}

	/**
	 * Constructor for decoding Raw.
	 * 
	 * @param x
	 * @param y
	 * @param w
	 * @param h
	 * @param pixelFormat
	 * @param input
	 * @throws IOException
	 * 
	 * @author Ken Breeman (kbreeman@wpi.edu)
	 * @author Eric Griffel (sniffels@wpi.edu)
	 */
	public Raw(int x, int y, int w, int h, PixelFormat pixelFormat,
			DataInputStream input) throws IOException {
		super(x, y, w, h);
		this.pixelFormat = pixelFormat;
		this.input = input;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gnu.rfb.Rect#writeData(java.io.DataOutput)
	 */
	public void writeData(DataOutput output) throws IOException {
		super.writeData(output);
		output.writeInt(RFB.EncodingRaw);
		output.write(bytes);
	}

	/**
	 * Reads in the data pre-emptively to decoding in. This is a performance
	 * enhancement to reduce the number of task switches.
	 * 
	 * @throws IOException
	 *             If unable to read from the input given in the constructor.
	 * 
	 * @author Ken Breeman (kbreeman@wpi.edu)
	 * @author Eric Griffel (sniffels@wpi.edu)
	 */
	public void readData() throws IOException {
		bytes = new byte[w * h * (pixelFormat.bitsPerPixel / 8)];
		int length = w * h * (pixelFormat.bitsPerPixel / 8);

		int offset = 0;
		int read = 0;
		while ((length - offset) > 0) {
			read = input.read(bytes, offset, (length - offset));
			if (read == -1) {
				try {
					Thread.sleep(1);
				} catch (InterruptedException e) {
					// Ignore this exception.
				}
			} else {
				offset += read;
			}
		}

	}

	/**
	 * Decodes from the input into an array of pixels as Integers.
	 * 
	 * @return The image as an array of pixels as Integers.
	 * @throws IOException
	 *             If unable to read from the input given in the constructor.
	 * 
	 * @author Ken Breeman (kbreeman@wpi.edu)
	 * @author Eric Griffel (sniffels@wpi.edu)
	 */
	public int[] decode() throws IOException {

		this.readData();
		// have bytes[]
		// Decode as pixels

		this.pixels = new int[w * h];
		int p = 0; // Index into the pixel array.

		switch (pixelFormat.bitsPerPixel) {
		case 32:

			for (int i = 0; i < (w * h * pixelFormat.bitsPerPixel / 8);) {
				// int pixel = input.readInt();

				byte b1 = bytes[i++];
				byte b2 = bytes[i++];
				byte b3 = bytes[i++];
				byte b4 = bytes[i++];

				int pixel = 0;
				pixel = (b4 & 0xFF);
				pixel = pixel << 8;
				pixel = pixel + (b3 & 0xFF);
				pixel = pixel << 8;
				pixel = pixel + (b2 & 0xFF);
				pixel = pixel << 8;
				pixel = pixel + (b1 & 0xFF);

				// de-translate pixel
				// ?pixelFormat.translatePixel(pixel);

				pixels[p++] = pixel;
			}
			break;

		case 16:

			for (int i = 0; i < (w * h * pixelFormat.bitsPerPixel / 8);) {
				// int pixel = input.readShort();

				byte b1 = bytes[i++];
				byte b2 = bytes[i++];

				int pixel = 0;
				pixel = pixel + (b2 & 0xFF);
				pixel = pixel << 8;
				pixel = pixel + (b1 & 0xFF);

				// de-translate pixel
				// ?pixelFormat.translatePixel(pixel);

				pixels[p++] = pixel;
			}
			break;

		case 8:

			for (int i = 0; i < (w * h * pixelFormat.bitsPerPixel / 8);) {
				// int pixel = input.readShort();

				byte b1 = bytes[i++];

				int pixel = 0;
				pixel = pixel + (b1 & 0xFF);

				// de-translate pixel
				// ?pixelFormat.translatePixel(pixel);

				pixels[p++] = pixel;
			}

			break;
		}

		return pixels;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gnu.rfb.Rect#clone()
	 */
	public Object clone() throws CloneNotSupportedException {
		return new Raw(x, y, w, h, pixelFormat, (byte[]) bytes.clone());
	}

}
