package gnu.rfb.client;

/**
 * <br><br><center><table border="1" width="80%">
 * 
 * <p>
 * Copyright (C) 2007-2008 VNCj
 * <p>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * <p>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * <a href="http://www.gnu.org/copyleft/lesser.html">GNU Lesser General Public License</a>
 * for more details.
 * <p>
 * You should have received a copy of the <a href="http://www.gnu.org/copyleft/lesser.html">
 * GNU Lesser General Public License</a> along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * <hr></table></center>
 **/

import gnu.rfb.CoRRE;
import gnu.rfb.Hextile;
import gnu.rfb.PixelFormat;
import gnu.rfb.RFB;
import gnu.rfb.RRE;
import gnu.rfb.Raw;

import java.io.DataInputStream;

/**
 * Takes care of reading in, decoding, and applying a FrameBuffer update
 * message.
 * 
 * @author Ken Breeman (kbreeman@wpi.edu)
 * @author Eric Griffel (sniffels@wpi.edu)
 */
public class FrameBufferUpdate {

	// The current PixelFormat to use.
	PixelFormat currentPixelFormat;

	// The input to read from.
	DataInputStream input;

	// Related to just this update
	int x;
	int y;
	int w;
	int h;
	int encoding;
	int pixels[];

	// Related to the entire frame buffer.
	int[] img;
	int width;
	int height;

	/**
	 * Default constructor.
	 * 
	 * @param pf
	 * @param dis
	 * @param img
	 * @param width
	 * @param height
	 * @throws Exception
	 */
	public FrameBufferUpdate(PixelFormat pf, DataInputStream dis, int[] img,
			int width, int height) throws Exception {
		this.currentPixelFormat = pf;
		this.input = dis;
		this.img = img;
		this.width = width;
		this.height = height;

		this.doUpdate();
	}

	/**
	 * Performs the update by reading in the data,
	 * decoding it using the appropriate encoding,
	 * and updates the pixel array. 
	 * 
	 * @throws Exception If unable to read from input or an error in the data occurs. 
	 */
	private void doUpdate() throws Exception {

		x = input.readShort() & 0xFFFF;
		y = input.readShort() & 0xFFFF;
		w = input.readShort() & 0xFFFF;
		h = input.readShort() & 0xFFFF;
		encoding = input.readInt();

		if ((encoding != RFB.EncodingRaw) && (encoding != RFB.EncodingRRE)
				&& (encoding != RFB.EncodingCoRRE)
				&& (encoding != RFB.EncodingCopyRect)
				&& (encoding != RFB.EncodingHextile)) {
			System.out
					.println("ENCODING NOT Raw, RRE, CoRRE, CopyRect, or Hextile!");
			throw new Exception("ERROR: Inavlid Encoding received.");
		}

		pixels = new int[w * h];

		switch (encoding) {
		case RFB.EncodingRaw:

			// System.out.println("RAW");
			Raw tempRaw = new Raw(x, y, w, h, currentPixelFormat, input);
			pixels = tempRaw.decode();
			break;

		case RFB.EncodingCopyRect:

			// System.out.println("CopyRect");
			// this.initialImgData.getPixels(x, y, w, pixels, 0);

			// TODO: FIX COPY RECT

			break;

		case RFB.EncodingRRE:

			// System.out.println("RRE");
			RRE tempRRE = new RRE(x, y, w, h, currentPixelFormat, input);
			pixels = tempRRE.decode();
			break;

		case RFB.EncodingCoRRE:

			// System.out.println("CoRRE");
			CoRRE tempCoRRE = new CoRRE(x, y, w, h, currentPixelFormat, input);
			pixels = tempCoRRE.decode();
			break;

		case RFB.EncodingHextile:

			// System.out.println("Hextile");
			Hextile tempHextile = new Hextile(x, y, w, h, currentPixelFormat,
					input);
			pixels = tempHextile.decode();
			break;

		default:
			System.err.println("ERORR: Invalid encoding received.");
		}

		this.updateImage();

	}

	/**
	 * Takes the previous frame buffer and updates it.
	 */
	private void updateImage() {

		// Take the Image stored in 'img' and update it with new pixels.
		for (int j = 0; j < h; j++) {
			for (int i = 0; i < w; i++) {
				this.img[((y + j) * width) + (x + i)] = this.pixels[(j * w) + i];
			}
		}

		return;
	}

	/**
	 * @return The image as an array of pixels (Integers)
	 */
	public int[] getImage() {
		return img;
	}
}
