package gnu.rfb.server;

/**
 * <br><br><center><table border="1" width="80%"><hr>
 * <strong><a href="http://www.amherst.edu/~tliron/vncj">VNCj</a></strong>
 * <p>
 * Copyright (C) 2000-2002 by Tal Liron
 * <p>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * <p>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * <a href="http://www.gnu.org/copyleft/lesser.html">GNU Lesser General Public License</a>
 * for more details.
 * <p>
 * You should have received a copy of the <a href="http://www.gnu.org/copyleft/lesser.html">
 * GNU Lesser General Public License</a> along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * <hr></table></center>
 **/

import gnu.rfb.Colour;
import gnu.rfb.PixelFormat;
import gnu.rfb.RFB;
import gnu.rfb.Rect;
import gnu.vnc.VLogger;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

public class RFBSocket implements RFBClient, Runnable {

	private Socket socket;
	private RFBHost host;
	private RFBAuthenticator authenticator;
	private RFBServer server = null;
	private DataInputStream input;
	private DataOutputStream output;

	private PixelFormat pixelFormat = null;
	private String protocolVersionMsg = "";
	private boolean shared = true;
	private int[] encodings = new int[0];
	private int preferredEncoding = RFB.EncodingHextile;
	private boolean isRunning = false;
	private boolean threadFinished = false;
	private Vector<UpdateRequest> updateQueue = new Vector<UpdateRequest>();
	private boolean updateAvailable = true;

	/**
	 * new constructor by Marcus Wolschon
	 */
	public RFBSocket(Socket socket, RFBServer server, RFBHost host,
			RFBAuthenticator authenticator) throws IOException {
		this.socket = socket;
		this.server = server;
		this.host = host;
		this.authenticator = authenticator;
		// Streams
		input = new DataInputStream(new BufferedInputStream(socket
				.getInputStream()));
		output = new DataOutputStream(new BufferedOutputStream(socket
				.getOutputStream(), 16384));

		// Start socket listener thread
		new Thread(this).start();
	}

	/**
	 * new constructor by Marcus Wolschon
	 */
	public RFBSocket(Socket socket, RFBServer server, RFBHost host,
			RFBAuthenticator authenticator, boolean syncronous)
			throws IOException {
		this.socket = socket;
		this.server = server;
		this.host = host;
		this.authenticator = authenticator;

		// Streams
		input = new DataInputStream(new BufferedInputStream(socket
				.getInputStream()));
		output = new DataOutputStream(new BufferedOutputStream(socket
				.getOutputStream(), 16384));

		// Start socket listener thread
		if (syncronous) {
			run();
		} else {
			new Thread(this).start();
		}
	}

	//
	// RFBClient
	//

	/*
	 * 
	 */
	public synchronized PixelFormat getPixelFormat() {
		return pixelFormat;
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBClient#getProtocolVersionMsg()
	 */
	public synchronized String getProtocolVersionMsg() {
		return protocolVersionMsg;
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBClient#getShared()
	 */
	public synchronized boolean getShared() {
		return shared;
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBClient#getPreferredEncoding()
	 */
	public synchronized int getPreferredEncoding() {
		return preferredEncoding;
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBClient#setPreferredEncoding(int)
	 */
	public synchronized void setPreferredEncoding(int encoding) {
		if (encodings.length > 0) {
			for (int i = 0; i < encodings.length; i++) {
				if (encoding == encodings[i]) {
					// Encoding is supported
					preferredEncoding = encoding;
					return;
				}
			}
		} else {
			// No list
			preferredEncoding = encoding;
		}
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBClient#getEncodings()
	 */
	public synchronized int[] getEncodings() {
		return encodings;
	}

	// Messages from server to client

	/**
	 * Writes the frame buffer update out to the client.
	 * 
	 * @param rects An array of rectangles to write.
	 */
	public synchronized void writeFrameBufferUpdate(Rect rects[])
			throws IOException {
		writeServerMessageType(RFB.FrameBufferUpdate);
		output.writeByte(0); // padding

		// Count rects
		int count = 0;
		int i;
		for (i = 0; i < rects.length; i++)
			count += rects[i].count;
		output.writeShort(count);

		for (i = 0; i < rects.length; i++)
			rects[i].writeData(output);

		output.flush();
	}

	/**
	 * Writes the color map entries to the client.
	 * 
	 * @param firstColour
	 * @param colours
	 * @throws IOException if unable to write the entries to the client.
	 */
	public synchronized void writeSetColourMapEntries(int firstColour,
			Colour colours[]) throws IOException {
		writeServerMessageType(RFB.SetColourMapEntries);
		output.writeByte(0); // padding
		output.writeShort(firstColour);
		output.writeShort(colours.length);
		for (int i = 0; i < colours.length; i++) {
			output.writeShort(colours[i].r);
			output.writeShort(colours[i].g);
			output.writeShort(colours[i].b);
		}
		output.flush();
	}

	/**
	 * Writes the bell message to the client.
	 * 
	 * @throws IOException If unable to write the message to the client.
	 */
	public synchronized void writeBell() throws IOException {
		writeServerMessageType(RFB.Bell);
	}

	/**
	 * Writes a server cut text message to the client.
	 */
	public synchronized void writeServerCutText(String text) throws IOException {
		writeServerMessageType(RFB.ServerCutText);
		output.writeByte(0); // padding
		output.writeShort(0); // padding
		output.writeInt(text.length());
		output.writeBytes(text);
		output.writeByte(0);
		output.flush();
	}

	// Operations

	/**
	 * Closes the connection to the client.
	 */
	public synchronized void close() {
		isRunning = false;
		// Block until the thread has exited gracefully
		while (threadFinished == false) {
			try {
				Thread.sleep(20);
			} catch (InterruptedException x) {
			}
		}
		try {
			socket.close();
		} catch (IOException e) {
			VLogger.getLogger().log(
					"Got and exception shutting down RFBSocket ", e);
		} finally {
			socket = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		isRunning = true;
		try {
			// System.err.println("DEBUG[RFBSocket] run() calling
			// writeProtocolVersionMsg()");
			// Handshaking
			writeProtocolVersionMsg();
			// System.err.println("DEBUG[RFBSocket] run() calling
			// readProtocolVersionMsg()");
			readProtocolVersionMsg();
			// System.err.println("DEBUG[RFBSocket] run() calling
			// writeAuthScheme()");
			// if(((DefaultRFBAuthenticator)authenticator).authenticate(input,output)==false){
			if (authenticator.authenticate(input, output, this) == false) {
//				System.out.println("Authentiation failed");
				return;
			}
			// System.err.println("DEBUG[RFBSocket] run() calling
			// readClientInit()");
			readClientInit();
			// System.err.println("DEBUG[RFBSocket] run() calling
			// initServer()");
			initServer();
			// System.err.println("DEBUG[RFBSocket] run() calling
			// writeServerInit()");
			writeServerInit();
			// System.err.println("DEBUG[RFBSocket] run() message loop");

			// //////////////////////////////////////////////////////////////////
			/* Modified by Ken Breeman (kbreeman@wpi.edu) to provide timely updates. */
			Date lastUpdate = new Date(); //Yes its a bit of a hack.
			// //////////////////////////////////////////////////////////////////

			// RFBClient message loop
			while (isRunning) {
				if (getUpdateIsAvailable()) {
					// go ahead and send the updates
					doFrameBufferUpdate();
				}
				if (input.available() == 0) {
					try {
						Thread.sleep(10);
					} catch (InterruptedException x) {
					}
				} else {
					switch (input.readUnsignedByte()) {
					case RFB.SetPixelFormat:
						readSetPixelFormat();
						break;
					case RFB.FixColourMapEntries:
						readFixColourMapEntries();
						break;
					case RFB.SetEncodings:
						readSetEncodings();
						break;
					case RFB.FrameBufferUpdateRequest:
						readFrameBufferUpdateRequest();
						break;
					case RFB.KeyEvent:
						readKeyEvent();
						break;
					case RFB.PointerEvent:
						readPointerEvent();
						break;
					case RFB.ClientCutText:
						readClientCutText();
						break;
					}
				}

				// ////////////////////////////////////////////////////////////
				/* Modified by Ken Breeman (kbreeman@wpi.edu) to provide timely updates.
				 * Yes it's a bit of a hack but it works.
				 */

				if (this.pixelFormat == null) {
					continue;
				}

				Date d = new Date();
				if (d.getTime() > (lastUpdate.getTime() + 1000)) {
//					System.out
//							.println(d.getTime() + " " + lastUpdate.getTime());
					lastUpdate = d;
					setUpdateIsAvailable(true);

					boolean incremental = false; // maybe false?
					int x = 0;
					int y = 0;
					int w = Math.max(1, server.getFrameBufferWidth(this));
					int h = Math.max(1, server.getFrameBufferHeight(this));
					UpdateRequest r = new UpdateRequest(incremental, x, y, w, h);
					synchronized (updateQueue) {
						int index = updateQueue.indexOf(r);
						if (index >= 0) {
							// replace only if update is non incremental
							if (r.incremental == false) {
								updateQueue.setElementAt(r, index);
							}
						} else {
							updateQueue.add(r);
						}
					}
				}
				// ///////////////////////////////////////////////////////////////////////////

			}
		} catch (IOException x) {
//			System.out.println("Got an IOException, drop the client");
		} catch (Throwable t) {
			t.printStackTrace();
		}

		if (server != null) {
			server.removeClient(this);
		}

		threadFinished = true;
		close();
	}

	/**
	 * Initializes the server.
	 * @throws IOException If unable to communicate with the client during the init.
	 */
	private void initServer() throws IOException {
		// We may already have a shared server
		if (shared) {
			server = host.getSharedServer();
		}
		server.addClient(this);
		server.setClientProtocolVersionMsg(this, protocolVersionMsg);
		server.setShared(this, shared);
	}

	/**
	 * Writes the protocol version message to the client.
	 * 
	 * @throws IOException If unable to write to the client.
	 */
	private synchronized void writeProtocolVersionMsg() throws IOException {
		output.writeBytes(RFB.ProtocolVersionMsg);
		output.flush();
	}

	/**
	 * Reads the protocol version message from the client.
	 * 
	 * @throws IOException If unable to write to the client.
	 */
	private synchronized void readProtocolVersionMsg() throws IOException {
		byte[] b = new byte[12];
		input.readFully(b);
		protocolVersionMsg = new String(b);
	}

	/**
	 * Reads the client init message.
	 * 
	 * @throws IOException If unable to write to the client.
	 */
	private synchronized void readClientInit() throws IOException {
		shared = input.readUnsignedByte() == 1;
	}

	/**
	 * Writes the servers init message.
	 * 
	 * @throws IOException If unable to write to the client.
	 */
	private synchronized void writeServerInit() throws IOException {
		// System.err.println("DEBUG[RFBSocket] writeServerInit() writing
		// FB-dimension");
		output.writeShort(server.getFrameBufferWidth(this));
		output.writeShort(server.getFrameBufferHeight(this));
		// System.err.println("DEBUG[RFBSocket] writeServerInit() writing
		// pixel-format");
		server.getPreferredPixelFormat(this).writeData(output);
		// System.err.println("DEBUG[RFBSocket] writeServerInit() writing
		// padding");
		output.writeByte(0); // padding
		output.writeByte(0); // padding
		output.writeByte(0); // padding
		// System.err.println("DEBUG[RFBSocket] writeServerInit() writing
		// desktopname");
		String desktopName = server.getDesktopName(this);
		output.writeInt(desktopName.length());
		output.writeBytes(desktopName);
		output.flush();
	}

	/**
	 * 
	 * @param type
	 * @throws IOException If unable to write to the client.
	 */
	private synchronized void writeServerMessageType(int type)
			throws IOException {
		output.writeByte(type);
	}

	/**
	 * Reads the Pixel Format message from the client.
	 * 
	 * @throws IOException If unable to read from the client.
	 */
	private synchronized void readSetPixelFormat() throws IOException {
		input.readUnsignedByte(); // padding
		input.readUnsignedShort(); // padding
		pixelFormat = new PixelFormat(input);
		input.readUnsignedByte(); // padding
		input.readUnsignedShort(); // padding

		// Delegate to server
		server.setPixelFormat(this, pixelFormat);
	}

	/**
	 * Reads the fix color map entries message from the client.
	 * 
	 * @throws IOException If unable to read from the client.
	 */
	private synchronized void readFixColourMapEntries() throws IOException {
		input.readUnsignedByte(); // padding
		int firstColour = input.readUnsignedShort();
		int nColours = input.readUnsignedShort();
		Colour colourMap[] = new Colour[nColours];
		for (int i = 0; i < nColours; i++)
			colourMap[i].readData(input);

		// Delegate to server
		server.fixColourMapEntries(this, firstColour, colourMap);
	}

	/**
	 * Reads the set encoding message(s) from the client.
	 * 
	 * @throws IOException If unable to read from the client.
	 */
	private synchronized void readSetEncodings() throws IOException {
		input.readUnsignedByte(); // padding
		int nEncodings = input.readUnsignedShort();
		encodings = new int[nEncodings];
		for (int i = 0; i < nEncodings; i++)
			encodings[i] = input.readInt();

		preferredEncoding = Rect.bestEncoding(encodings);

		// Delegate to server
		server.setEncodings(this, encodings);
	}

	/**
	 * Reads in a frame buffer update request from the client.
	 * 
	 * @throws IOException If unable to read from the client.
	 */
	private synchronized void readFrameBufferUpdateRequest() throws IOException {
		boolean incremental = (input.readUnsignedByte() == 1);
		int x = input.readUnsignedShort();
		int y = input.readUnsignedShort();
		int w = input.readUnsignedShort();
		int h = input.readUnsignedShort();
		UpdateRequest r = new UpdateRequest(incremental, x, y, w, h);
		synchronized (updateQueue) {
			int index = updateQueue.indexOf(r);
			if (index >= 0) {
				// replace only if update is non incremental
				if (r.incremental == false) {
					updateQueue.setElementAt(r, index);
				}
			} else {
				updateQueue.add(r);
			}
		}
	}

	/**
	 * Writes a frame buffer update message to the client.
	 * 
	 * @throws IOException If unable to write to the client.
	 */
	private synchronized void doFrameBufferUpdate() throws IOException {
		Iterator<UpdateRequest> iter = updateQueue.iterator();
		while (iter.hasNext()) {
			UpdateRequest ur = (UpdateRequest) iter.next();
			iter.remove();
			// Delegate to server
			try {
//				System.out.println("RFBSocket is doing an update");
				server.frameBufferUpdateRequest(this, ur.incremental, ur.x,
						ur.y, ur.w, ur.h);
//				System.out.println("RFBSocket is done");
			} catch (IOException e) // some times we have w==h==0 and it would
									// result in a blue screen on the official
									// VNC client.
			{
				// if there is nothing to encode, encode the top left pixel
				// instead
				if (e.getMessage().startsWith("rects.length == 0")) {
					server.frameBufferUpdateRequest(this, false, 0, 0, 1, 1);
				} else {
					// rethrow it
					throw e;
				}
			} finally {
				setUpdateIsAvailable(false);
			}
		}
	}

	/**
	 * Reads in a key event message from the client.
	 * 
	 * @throws IOException If unable to read from the client.
	 */
	private synchronized void readKeyEvent() throws IOException {
		boolean down = (input.readUnsignedByte() == 1);
		input.readUnsignedShort(); // padding
		int key = input.readInt();

		// Delegate to server
		server.keyEvent(this, down, key);
	}

	/**
	 * Reads in a mouse event from the client.
	 * 
	 * @throws IOException If unable to read from the client.
	 */
	private synchronized void readPointerEvent() throws IOException {
		int buttonMask = input.readUnsignedByte();
		int x = input.readUnsignedShort();
		int y = input.readUnsignedShort();

		// Delegate to server
		server.pointerEvent(this, buttonMask, x, y);
	}

	/**
	 * Reads in a client cut text message from the client.
	 * 
	 * @throws IOException If unable to read from the client.
	 */
	private synchronized void readClientCutText() throws IOException {
		input.readUnsignedByte(); // padding
		input.readUnsignedShort(); // padding
		int length = input.readInt();
		byte[] bytes = new byte[length];
		input.readFully(bytes);
		String text = new String(bytes);

		// Delegate to server
		server.clientCutText(this, text);
	}

	/**
	 * @return The InetAddress of the client.
	 */
	public InetAddress getInetAddress() {
		return (socket.getInetAddress());
	}

	/**
	 * @return The name of the host as a String.
	 */
	public String getName() {
		return (host.getDisplayName());
	}

	/**
	 * Set whether or not an update is available to send.
	 */
	public void setUpdateIsAvailable(boolean value) {
		updateAvailable = value;
	}

	/**
	 * @return Whether or not an update is available to send.
	 */
	public boolean getUpdateIsAvailable() {
		return (updateAvailable);
	}

	/**
	 * Represents a simplified update request message. 
	 */
	private class UpdateRequest {
		boolean incremental;
		int x;
		int y;
		int w;
		int h;

		/**
		 * Default constructor.
		 * 
		 * @param incremental
		 * @param x
		 * @param y
		 * @param w
		 * @param h
		 */
		public UpdateRequest(boolean incremental, int x, int y, int w, int h) {
			this.incremental = incremental;
			this.x = x;
			this.y = y;
			this.w = w;
			this.h = h;
		}

		/*
		 * (non-Javadoc)
		 * @see java.lang.Object#equals(java.lang.Object)
		 */
		public boolean equals(Object obj) {
			UpdateRequest u2 = (UpdateRequest) obj;
			return (x == u2.x && y == u2.y && w == u2.w && h == u2.h);
		}
	}
}
