package gnu.vnc.awt;

/**
 * <br><br><center><table border="1" width="80%"><hr>
 * <strong><a href="http://www.amherst.edu/~tliron/vncj">VNCj</a></strong>
 * <p>
 * Copyright (C) 2000-2002 by Tal Liron
 * <p>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * <p>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * <a href="http://www.gnu.org/copyleft/lesser.html">GNU Lesser General Public License</a>
 * for more details.
 * <p>
 * You should have received a copy of the <a href="http://www.gnu.org/copyleft/lesser.html">
 * GNU Lesser General Public License</a> along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * <hr></table></center>
 **/

import gnu.rfb.Colour;
import gnu.rfb.PixelFormat;
import gnu.rfb.Rect;
import gnu.rfb.KeySym;
import gnu.rfb.RFB;
import gnu.rfb.server.RFBClient;
import gnu.rfb.server.RFBServer;

import java.awt.AWTException;
import java.awt.Component;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.awt.image.DirectColorModel;
import java.io.IOException;
import java.util.ArrayList;

public class VNCRobot extends Component implements RFBServer {

	/* The name of the display. */
	private String displayName;
	
	/* The graphics device to take screenshots from. */
	private GraphicsDevice device;
	
	/* The Java Robot to control keyboard, mouse, and screen capture. */
	private Robot robot;
	
	/* The last known state of the mouse. */
	private int mouseModifiers = 0;

	/* Default serial version id. */
	private static final long serialVersionUID = 1L;


	/**
	 * Constructs a new VNC Robot to control keyboard and mouse as well as capture screen shots. 
	 * @param display The number for the display.
	 * @param displayName The name of the display.
	 */
	public VNCRobot(int display, String displayName) {
		this.displayName = displayName;
		device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
		try {
			robot = new Robot(device);
		} catch (AWTException x) {
			//System.out.println("Caught AWT Exception!!!");
		}
	}


	//
	// RFBServer
	//

	// Clients

	/*
	 * 
	 */
	public void addClient(RFBClient client) {
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#removeClient(gnu.rfb.server.RFBClient)
	 */
	public void removeClient(RFBClient client) {
	}

	// Attributes

	/*
	 * 
	 */
	public String getDesktopName(RFBClient client) {
		return displayName;
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#getFrameBufferWidth(gnu.rfb.server.RFBClient)
	 */
	public int getFrameBufferWidth(RFBClient client) {
		// return 200;
		return device.getDefaultConfiguration().getBounds().width;
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#getFrameBufferHeight(gnu.rfb.server.RFBClient)
	 */
	public int getFrameBufferHeight(RFBClient client) {
		// return 200;
		return device.getDefaultConfiguration().getBounds().height;
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#getPreferredPixelFormat(gnu.rfb.server.RFBClient)
	 */
	public PixelFormat getPreferredPixelFormat(RFBClient client) {
		return PixelFormat.RGB888;
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#allowShared()
	 */
	public boolean allowShared() {
		return true;
	}

	// Messages from client to server

	/**
	 * 
	 */
	public void setClientProtocolVersionMsg(RFBClient client, String protocolVersionMsg) throws IOException {
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#setShared(gnu.rfb.server.RFBClient, boolean)
	 */
	public void setShared(RFBClient client, boolean shared) throws IOException {
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#setPixelFormat(gnu.rfb.server.RFBClient, gnu.rfb.PixelFormat)
	 */
	public void setPixelFormat(RFBClient client, PixelFormat pixelFormat) throws IOException {
		pixelFormat.setDirectColorModel((DirectColorModel) Toolkit.getDefaultToolkit().getColorModel());
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#setEncodings(gnu.rfb.server.RFBClient, int[])
	 */
	public void setEncodings(RFBClient client, int[] encodings) throws IOException {
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#fixColourMapEntries(gnu.rfb.server.RFBClient, int, gnu.rfb.Colour[])
	 */
	public void fixColourMapEntries(RFBClient client, int firstColour, Colour[] colourMap) throws IOException {
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#frameBufferUpdateRequest(gnu.rfb.server.RFBClient, boolean, int, int, int, int)
	 */
	public void frameBufferUpdateRequest(RFBClient client, boolean incremental, int x, int y, int w, int h) throws IOException {
		
		if (incremental) {
			return;
		}

		// Create image
		BufferedImage image = robot.createScreenCapture(new Rectangle(x, y, w, h));

		// Encode image
		Rect r = Rect.encode(client.getPreferredEncoding(), client.getPixelFormat(), image, x, y, w, h);

		// Write to client
		Rect[] rects = { r };
		client.writeFrameBufferUpdate(rects);
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#keyEvent(gnu.rfb.server.RFBClient, boolean, int)
	 */
	public void keyEvent(RFBClient client, boolean down, int key) throws IOException {
		
		int vk = KeySym.toVKall(key);
		
		if (vk > 0) {
			//If we have a known key.
			
			if (down) {
				//System.out.println("key-press: " + key + " " + vk);
				robot.keyPress(vk);
			} else {
				//System.out.println("key-rel:" + key + " " + vk);
				robot.keyRelease(vk);
			}

		} else if (vk == -1) {
			// If we have a combo key such as:   +_)(*&^%$#@!~|}{":?><")
			
			if(down) {
				//Key down
				ArrayList keys = KeySym.VKcombo(key);
				for(int i=0; i<keys.size(); i++) {
					//System.out.println("combo-key-press: " + key + " " + vk);
					robot.keyPress((Integer) (keys.get(i)));
				}
				return;
			} else {
				//Key up
				ArrayList keys = KeySym.VKcombo(key);
				for(int i=0; i<keys.size(); i++) {
					//System.out.println("combo-key-rel: " + key + " " + vk);
					robot.keyRelease((Integer) (keys.get(i)));
				}
			}

		} else {
			//Else we have an unknown key.
			//System.out.println("Unknown key: " + key + "down? " + down);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#pointerEvent(gnu.rfb.server.RFBClient, int, int, int)
	 */
	public void pointerEvent(RFBClient client, int buttonMask, int x, int y) throws IOException {
		// Modifiers
		int newMouseModifiers = 0;
		
		if ((buttonMask & RFB.Button1Mask) != 0) {
			newMouseModifiers |= MouseEvent.BUTTON1_MASK;
		}
		if ((buttonMask & RFB.Button2Mask) != 0) {
			newMouseModifiers |= MouseEvent.BUTTON2_MASK;
		}
			
		if ((buttonMask & RFB.Button3Mask) != 0) {
			newMouseModifiers |= MouseEvent.BUTTON3_MASK;
		}

		if (newMouseModifiers != mouseModifiers) {
			// Change of button state
			if (mouseModifiers == 0) {
				robot.mousePress(newMouseModifiers);
			} else {
				//Release the old buttons clicked.
				robot.mouseRelease(mouseModifiers);
			}

			mouseModifiers = newMouseModifiers;
		}

		robot.mouseMove(x, y);
	}

	/*
	 * (non-Javadoc)
	 * @see gnu.rfb.server.RFBServer#clientCutText(gnu.rfb.server.RFBClient, java.lang.String)
	 */
	public void clientCutText(RFBClient client, String text) throws IOException {
	}

	/**
	 * 
	 * @param client
	 * @return
	 */
	public boolean isUpdateAvailable(RFBClient client) {
		return false;
	}
	
}
